# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/04_source_to_refined.ipynb (unless otherwise specified).

__all__ = ['NifiIds', 'source_to_refined']

# Cell

import json
from os import environ
from time import sleep
from .dataflow import DataFlow
from .environment import DataFlowIds

# Cell
class NifiIds:
    """ The collection of the DataFlowIds:
          - source to raw
          - raw to discovery
          - discovery to refined
    """

    nifi_json_ids = json.load(open(environ['CONFIG_FILE']))['NifiIds']

    sourceToRaw = DataFlowIds(pipeline=nifi_json_ids['SourceToRaw'])
    rawToDiscovery = DataFlowIds(pipeline=nifi_json_ids['RawToDiscovery'])
    discoveryToRefined = DataFlowIds(
        pipeline=nifi_json_ids['DiscoveryToRefined'])

# Cell
def source_to_refined():
    """ The Source To Refined dataflow is decomposed in three sequential steps:
        - Source To Raw
        - Raw To Discovery
        - Discovery To Refined
    This function assures that each of these steps starts only after the
    previous step has finished.
    """

    # time between steps
    T = 15
    time_after_start = 15

    time_between_checks = 15

    source_to_raw = DataFlow(
        NifiIds.sourceToRaw,
        delay_seconds_after_start=time_after_start,
        delay_seconds_between_checks=time_between_checks,
    )
    raw_to_discovery = DataFlow(
        NifiIds.rawToDiscovery,
        delay_seconds_after_start=time_after_start,
        delay_seconds_between_checks=time_between_checks,
    )
    discovery_to_refined = DataFlow(
        NifiIds.discoveryToRefined,
        delay_seconds_after_start=time_after_start,
        delay_seconds_between_checks=time_between_checks,
    )

    source_to_raw.run()
    sleep(T)
    raw_to_discovery.run()
    sleep(T)
    discovery_to_refined.run()


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/01_environment.ipynb (unless otherwise specified).

__all__ = ['Credentials', 'NifiEndpoint', 'DataFlowIds']

# Cell
import requests
from os import environ
import json

# Cell
class Credentials:

    user = environ['CLOUDERA_USER']
    password = environ['CLOUDERA_PASS']
    credentials = (user, password)

# Cell
class NifiEndpoint:

    cluster = environ['CLOUDERA_CLUSTER']
    nifi_rest = cluster + environ['CLOUDERA_NIFI_REST']
    processors = nifi_rest + "processors/"
    connections = nifi_rest + "connections/"
    flowfile_queues = nifi_rest + "flowfile-queues/"

# Cell


class DataFlowIds:
    """ Dataflow uuids. Every valid DataFlow must have initial-middle-final
        procesors and initial-final connections
        """

    def __init__(
        self,
        pipeline: dict,
    ) -> None:
        self.in_connection = pipeline['in_connection']['Id']
        self.in_processor = pipeline['in_processor']['Id']
        self.middle_processor = pipeline['middle_processor']['Id']
        self.out_connection = pipeline['out_connection']['Id']
        self.out_processor = pipeline['out_processor']['Id']





# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"Credentials": "01_environment.ipynb",
         "NifiEndpoint": "01_environment.ipynb",
         "DataFlowIds": "01_environment.ipynb",
         "Processor": "02_rest.ipynb",
         "Flowfiles": "02_rest.ipynb",
         "DataFlow": "03_dataflow.ipynb",
         "NifiIds": "04_source_to_refined.ipynb",
         "source_to_refined": "04_source_to_refined.ipynb",
         "custom_response": "09_tools.ipynb",
         "is_valid_csv": "09_tools.ipynb",
         "get_valid_names_in_connector": "09_tools.ipynb"}

modules = ["environment.py",
           "rest.py",
           "dataflow.py",
           "source_to_refined.py",
           "tools.py"]

doc_url = "https://kikejimenez.github.io/nifi_api/"

git_url = "https://github.com/kikejimenez/nifi_api/tree/main/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/03_dataflow.ipynb (unless otherwise specified).

__all__ = ['DataFlow']

# Cell

import time

from nifi_api.rest import Flowfiles, Processor

# Cell


class DataFlow:
    """
    Monitors and controls a Nifi dataflow. The dataflow starts
    when the **run** method is called.

    Parameters
   -------------

      dataFlowIds: DataFlowIds
        data structure that contains all the IDs of the in/out
        processors and connections

"""

    def __init__(
        self,
        dataflow_ids: object,
        delay_seconds_after_start: int = 14,
        delay_seconds_between_checks: int = 15,
    ) -> None:
        self.in_processor = Processor(dataflow_ids.in_processor)
        self.in_flowfiles = Flowfiles(dataflow_ids.in_connection)
        self.middle_processor = Processor(dataflow_ids.middle_processor)
        self.out_processor = Processor(dataflow_ids.out_processor)
        self.out_flowfiles = Flowfiles(dataflow_ids.out_connection)

        self.seconds_after_start = delay_seconds_after_start
        self.seconds_between_checks = delay_seconds_between_checks

    def run(self) -> None:

        print('pipeline watching has started..')

        self.out_processor.update_run_status("STOPPED")
        self.in_processor.update_run_status("RUNNING")
        time.sleep(self.seconds_after_start)
        self.in_flowfiles.get_ids()
        self.middle_processor.update_run_status("RUNNING")
        self.in_processor.update_run_status("STOPPED")

        while True:

            self.out_flowfiles.get_ids()

            if self.in_flowfiles.equals(self.out_flowfiles):

                self.middle_processor.update_run_status("STOPPED")
                self.out_processor.update_run_status("RUNNING")
                print("Pipeline watching has finished ...")
                break
            time.sleep(self.seconds_between_checks)


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/02_rest.ipynb (unless otherwise specified).

__all__ = ['Processor', 'Flowfiles']

# Cell

import requests
import json
from nifi_api.environment import NifiEndpoint, Credentials
from nifi_api.tools import custom_response, get_valid_names_in_connector

# Cell


class Processor:

    nifi_api = NifiEndpoint.processors

    def __init__(self, processor_id: str) -> None:
        self.processor_id = processor_id

    def get(self) -> tuple:
        url = self.nifi_api + self.processor_id
        res = requests.get(url, auth=Credentials.credentials)
        return custom_response(res)

    def get_state(self) -> None:
        self.state, _ = self.get()
        self.version = self.state['revision']['version']

    def update_run_status(self, state) -> tuple:
        """
Change the status of the processor.

 Attributes
----------
state : str
     The possible values are: "RUNNING", "STOPPED", "DISABLED".

 Returns
----------
 custom_response : tuple
   The response as defined in tools module
"""

        self.get_state()

        data = {
            "revision": {
                #   'clientId': self.client_id,
                'version': self.version
            },
            "state": state,
            "disconnectedNodeAcknowledged": True
        }

        url = self.nifi_api + f"{self.processor_id}/run-status"

        res = requests.put(url,
                           data=json.dumps(data),
                           auth=Credentials.credentials,
                           headers={'content-type': 'application/json'})
        return custom_response(res)

# Cell


class Flowfiles:

    def __init__(self, connection_id: str) -> None:
        self.connection_id = connection_id

    def post_listing_request(self) -> None:
        url = NifiEndpoint.flowfile_queues + f"{self.connection_id}/listing-requests"
        res = requests.post(url, auth=Credentials.credentials)
        self.listing_request = custom_response(res)[0]
        self.listing_request_id = self.listing_request["listingRequest"]['id']

    def get_list_queue(self) -> None:
        self.post_listing_request()
        endpoint = f"{self.connection_id}/listing-requests/{self.listing_request_id}"
        url = NifiEndpoint.flowfile_queues + endpoint
        res = requests.get(url, auth=Credentials.credentials)
        self.list_queue, _ = custom_response(res)

    def get_ids(self) -> None:
        self.get_list_queue()
        self.ids = get_valid_names_in_connector(self.list_queue)

    def equals(self, flowfile) -> bool:
        return set(self.ids) == set(flowfile.ids)


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/09_tools.ipynb (unless otherwise specified).

__all__ = ['custom_response', 'is_valid_csv', 'get_valid_names_in_connector']

# Cell
import json

# Cell

def custom_response(res):
    """ De-Serializes the JSON response
    """
    try:
        return json.loads(res.text), res
    except:
        return "No JSON available", res

# Cell


def is_valid_csv(file):
    """ A valid file is a csv file whose name is in a
    set of valid names

    """
    valid_names = [
        'agents',
        'calls',
        'summary',
        'skills',
        'teams',
        'campaings',
    ]

    filename = file.split('/')[-1].split('.')
    if len(filename) == 1:
        return False

    fname, ext = filename
    is_valid_name = any(vname in fname for vname in valid_names)
    return is_valid_name and ext.lower() == 'csv'

# Cell


def get_valid_names_in_connector(list_queues):
    """ Gets a list of valid names from the list of queues
    """
    flowfile_summaries = list_queues['listingRequest']['flowFileSummaries']
    valid_uuids = [
        ff['filename']
        for ff in flowfile_summaries
        if is_valid_csv(ff['filename'])
    ]
    return valid_uuids